package com.xunmei.system.controller;

import com.xunmei.common.core.domain.R;
import com.xunmei.common.core.utils.StringUtils;
import com.xunmei.common.core.utils.file.FileTypeUtils;
import com.xunmei.common.core.utils.file.MimeTypeUtils;
import com.xunmei.common.core.web.controller.BaseController;
import com.xunmei.common.core.web.domain.AjaxResult;
import com.xunmei.common.log.annotation.Log;
import com.xunmei.common.log.enums.BusinessType;
import com.xunmei.common.security.service.TokenService;
import com.xunmei.common.security.utils.SaltHelper;
import com.xunmei.common.security.utils.SecurityUtils;
import com.xunmei.system.api.RemoteFileService;
import com.xunmei.system.api.domain.SysFile;
import com.xunmei.system.api.domain.SysUser;
import com.xunmei.system.api.model.LoginUser;
import com.xunmei.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;

/**
 * 个人信息 业务处理
 * 
 * @author xunmei
 */
@RestController
@RequestMapping("/user/profile")
public class SysProfileController extends BaseController
{
    @Autowired
    private ISysUserService userService;
    
    @Autowired
    private TokenService tokenService;
    
    @Autowired
    private RemoteFileService remoteFileService;

    /**
     * 个人信息
     */
    @GetMapping
    public AjaxResult profile()
    {
        Long userId = SecurityUtils.getUserId();
        SysUser user = userService.selectUserById(userId);
        AjaxResult ajax = AjaxResult.success(user);
        ajax.put("roleGroup", userService.selectUserRoleGroup(user.getUsername()));
//        ajax.put("postGroup", userService.selectUserPostGroup(username));
        ajax.put("user", user);
        return ajax;
    }

    /**
     * 修改用户
     */
    @Log(title = "个人信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult updateProfile(@RequestBody SysUser user)
    {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        SysUser sysUser = loginUser.getSysUser();
        user.setUsername(sysUser.getUsername());
        if (StringUtils.isNotEmpty(user.getPhone()) && !userService.checkPhoneUnique(user))
        {
            return error("修改用户'" + user.getUsername() + "'失败，手机号码已存在");
        }
//        else if (StringUtils.isNotEmpty(user.ge()) && !userService.checkEmailUnique(user))
//        {
//            return error("修改用户'" + user.getUserName() + "'失败，邮箱账号已存在");
//        }
        user.setId(sysUser.getId());
        user.setPassword(null);
        user.setDeptId(null);
        if (userService.updateUserProfile(user) > 0)
        {
            // 更新缓存用户信息
            loginUser.getSysUser().setUsername(user.getUsername());
            loginUser.getSysUser().setPhone(user.getPhone());
            loginUser.getSysUser().setGender(user.getGender());
            loginUser.getSysUser().setImage(user.getImage());
            tokenService.setLoginUser(loginUser);
            return success();
        }
        return error("修改个人信息异常，请联系管理员");
    }

    /**
     * 重置密码
     */
    @Log(title = "个人信息", businessType = BusinessType.UPDATE)
    @PutMapping("/updatePwd")
    public AjaxResult updatePwd(String oldPassword, String newPassword)
    {
        Long userId = SecurityUtils.getUserId();
        SysUser user = userService.selectUserById(userId);
        String password = user.getPassword();
        if (!SecurityUtils.isEquals(password, oldPassword,user.getSalt()))
        {
            return error("修改密码失败，旧密码错误");
        }
        if (SecurityUtils.isEquals(password, newPassword,user.getSalt()))
        {
            return error("新密码不能与旧密码相同");
        }
        String salt = user.getSalt();
        String nPassword = SaltHelper.exec(newPassword, salt);
        if (userService.resetUserPwd(user.getUsername(),nPassword ) > 0)
        {
            // 更新缓存用户密码
            LoginUser loginUser = SecurityUtils.getLoginUser();
            loginUser.getSysUser().setPassword(nPassword);
            tokenService.setLoginUser(loginUser);
            return success();
        }
        return error("修改密码异常，请联系管理员");
    }
    
    /**
     * 头像上传
     */
    @Log(title = "用户头像", businessType = BusinessType.UPDATE)
    @PostMapping("/avatar")
    public AjaxResult avatar(@RequestParam("avatarfile") MultipartFile file)
    {
        if (!file.isEmpty())
        {
            LoginUser loginUser = SecurityUtils.getLoginUser();
            String extension = FileTypeUtils.getExtension(file);
            if (!StringUtils.equalsAnyIgnoreCase(extension, MimeTypeUtils.IMAGE_EXTENSION))
            {
                return error("文件格式不正确，请上传" + Arrays.toString(MimeTypeUtils.IMAGE_EXTENSION) + "格式");
            }
            R<SysFile> fileResult = remoteFileService.upload(file);
            if (StringUtils.isNull(fileResult) || StringUtils.isNull(fileResult.getData()))
            {
                return error("文件服务异常，请联系管理员");
            }
            String url = fileResult.getData().getUrl();
            if (userService.updateUserAvatar(loginUser.getUsername(), url))
            {
                AjaxResult ajax = AjaxResult.success();
                ajax.put("imgUrl", url);
                // 更新缓存用户头像
                loginUser.getSysUser().setImage(url);
                tokenService.setLoginUser(loginUser);
                return ajax;
            }
        }
        return error("上传图片异常，请联系管理员");
    }
}
